% Example physics data processing pipeline: #1 Pendulum
% * Load raw data from an Excel sheet pendulum.xlsx. This has columns of
%   pendulum length L /m, and three repeats of ten periods (10*T) /s.
% * Determine averages and errors
% * Plot y = 4*pi^2 * L vs x =T^2. Determine line of best fit (LOBF) and error,
%   and hence determine g from data. Compare to g = 9.81N/kg.
% * Plot T (data) vs 2*pi*sqrt(L/g) (with actual g). Perform LOBF.
% * Underlay T vs L data and underlay with T = 2*pi*sqrt(L/g) model.
%
% LAST UPDATED by Dr Andrew French. July 2020.

function pendulum

%% INPUTS %%

%Fontsize and marker size for graphs
fsize = 18; msize = 18;

%Set (fixed) error (in m) for pendulum length. Assume no systematic error.
Lerror = 0.01;

%Actual value of g /Nkg^-1
g = 9.81;

%Leave figures or close after printing?
close_after_print = 1;

%

%% IMPORT EXCEL DATA & PREPARE L, T arrays %%

%Import data. Four columns. First is pendulum length, next three are
% ten periods /s.
[num,txt,raw] = xlsread( 'pendulum' );
L = num(:,1); T10_1 = num(:,2); T10_2 = num(:,3); T10_3 = num(:,4);

%Determine period T /s and the (unbiased estimator) of the error in T.
%The second argument of the std function uses the /(N-1) normalization
T = mean( [T10_1 , T10_2 , T10_3 ],2 )/10;
Terror = std( [T10_1 , T10_2 , T10_3 ],0,2 )/10;

%

%% ANALYSIS: Compare T vs L data to model T(L) with actual g %%

%Determine model prediction of T using actual value of g
Tmodel = 2*pi*sqrt( L/g );

%Determine model at a much finer grid of L values
LL = linspace( 0,max(L),1000 ); TTmodel = 2*pi*sqrt( LL/g );

%Plot model curve of T vs L
figure('name','model vs data','color',[1 1 1],...
    'units','normalized','position',[0.05, 0.05, 0.9, 0.85]);
plot( LL, TTmodel,'b-','linewidth',2 ); hold on;
set( gca, 'fontsize',fsize ); grid on; 

%Plot data error bars
x = L; y = T; yneg = Terror; ypos = Terror;
xneg = Lerror*ones(size(L)); xpos = Lerror*ones(size(L)); 
errorbar( x,y,yneg,ypos,xneg,xpos,'o','color','r');

%Graph labels etc
xlabel('Pendulum length L /m'); ylabel('Pendulum period T /s');
title('Pendulum data underlaid with model using g=9.81N/kg');
legend({'Model','Data'});

%Print a PNG file
print( gcf, 'data underlaid with model.png','-r300','-dpng' );
if close_after_print==1; close(gcf); end

%

%% ANALYSIS: Determine line of best fit of the form y = m*x between T data and T model
% For 100% correlation, the gradient m = 1 and product-moment correlation coefficient r = 1.
y = Tmodel; x = T; [yfit,xfit,r,m,dm,yupper,ylower,s] = bestfit( x,y );

%Plot line of best fit
xlabel_str = 'Period T data /s';
ylabel_str = 'Period T model = 2\pi*sqrt(L/g) /s';
plot_LOBF( x,y, yfit,xfit,r,m,dm,yupper,ylower,...
    fsize, msize, xlabel_str, ylabel_str );

%Plot y = x for visual check
plot( [0;x], [0;x], 'm-', 'linewidth', 1 );
legend({'Lfit', 'Lfit upper','Lfit lower','x,y data','y = x'},...
    'location','southeast'); axis equal; axis tight;

%Set sensible x,y limits to include origin
xlimits = get(gca,'xlim'); set( gca, 'xlim',[0,round( xlimits(2) )] );
ylimits = get(gca,'ylim'); set( gca, 'ylim',[0,round( ylimits(2) )] );
print( gcf, 'model vs data.png','-r300','-dpng' );
if close_after_print==1; close(gcf); end

%

%% ANALYSIS: Determine g from data %%

%Determine y = 4*pi^2*L and x = T^2
x = T.^2; y = 4*pi^2 * L;

%Determine upper and lower values for error bar calculation
x_upper = ( T + Terror ).^2;  x_lower = ( T - Terror ).^2; 
y_upper = 4*pi^2 *( L + Lerror );  y_lower = 4*pi^2 *( L - Lerror ); 

% Determine line of best fit of the form y = m*x. 
% Gradient m is g in this case
[yfit,xfit,r,m,dm,yupper,ylower,s] = bestfit(x,y);

%Plot line of best fit
xlabel_str = '(T/s)^2'; ylabel_str = '4\pi^2*(L/m)';
plot_LOBF( x,y, yfit,xfit,r,m,dm,yupper,ylower,...
    fsize, 0.001, xlabel_str, ylabel_str );

%Plot what the line should be, given the actual value of g
plot( x, g*x, 'm-','linewidth',1 );

%Plot data error bars
yneg = y - y_lower; ypos = y_upper - y; xneg = x - x_lower; xpos = x_upper - x;
errorbar( x,y,yneg,ypos,xneg,xpos,'o','color','r');

%Add a legend
legend({'Lfit', 'Lfit upper','Lfit lower','',...
    'Using g=9.81N/kg','x,y data'},'location','southeast' )

%Print PNG
print( gcf, 'g from LOBF.png','-r300','-dpng' );
if close_after_print==1; close(gcf); end


%%

%Line of best fit function yfit = m*x, with product moment correlation
%coefficient r
function [yfit,xfit,r,m,dm,yupper,ylower,s] = bestfit(x,y)

%Find any x or y values that are NaN or Inf
ignore = isnan(abs(x)) | isnan(abs(y)) | isinf(abs(x)) | isinf(abs(y));
x(ignore) = []; y(ignore) = [];

%Compute line of best fit
xbar = mean(x); ybar = mean(y); xybar = mean(x.*y);
xxbar = mean(x.^2 ); yybar = mean(y.^2 );
Vx = xxbar - xbar^2; Vy = yybar - ybar^2;
COVxy = xybar - xbar*ybar;
m = xybar/xxbar; r = COVxy/sqrt( Vx*Vy );
[x,i] = sort(x); y = y(i);
yfit = m*x; xfit = x;

%Compute error in gradient m
n = length(x); s = sqrt( (1/(n-1))*sum( (y - yfit).^2 ) );
dm = s/sqrt(n*Vx);

%Determine envelope lines
yupper = (m+dm)*x; ylower = (m-dm)*x;

%%

%Plot line of best fit
function plot_LOBF( x,y, yfit,xfit,r,m,dm,yupper,ylower,...
    fsize, msize, xlabel_str, ylabel_str  )
figure('name','line of best fit','color',[1 1 1],...
    'units','normalized','position',[0.05, 0.05, 0.9, 0.85]);
plot( xfit, yfit, 'k-', xfit, yupper, 'b--', xfit, ylower, 'g--' );
hold on; plot( x,y,'r+','markersize',msize); set(gca, 'fontsize',fsize );
xlabel(xlabel_str); ylabel(ylabel_str); grid on;
title( {['Line of best fit y = ( ',num2str(m,3),' +/- ',num2str(dm,3),...
    ' )x, r = ',num2str(r,4)] } );

%End of code